#pragma once

#include "common.h"
#include "mpxerrors.h"

#define MPXMGR_NAME             "mpxmgr"        // name of mpx manager for accessing callbacks registered by manager itself
#define MPXMGR_CB_EXIT          "exit"          // callback event name for exit event (application is shutting down, but all is still functional (message processing, mpxCtrl library still live,...)
                                                // good time for clean up if cleanup requires win message processing
#define MPXMGR_CB_START         "start"         // callback event name for start event (every plugin was loaded)

#define MPXCTRL_NAME            "mpxctrl"       // "plugin name" for access callback events registered by MpxCtrl library
#define MPXCTRL_CB_ACQCOMPL     "acqCompleted"  // callback event name for acquisition completition
#define MPXCTRL_CB_INFOMSG      "infoMsg"       // callback event name for info/error messages
#define MPXCTRL_CB_DACSSET      "DACs set"      // DACs were set (changed)

#define NAME_LENGTH               32    // maximum length of registered names for plugins, callback events, function names... (in ExtFunctionInfo,...)
#define DESC_LENGTH               64    // maximum length of description string for plugins, callback events, function names... (in ExtFunctionInfo,...)
#define MENU_LENGTH              256    // maximum length of menu item path (path1|path2|item)
#define MAX_PARAM                  8    // maximu number of input/output parameters (limits number of ExtParamInfo in ExtFunctionInfo)
#define INVALID_DEVID_VALUE MAXDWORD    // invalid Device ID (DEVID type)
#define INVALID_ITEMID_VALUE       0    // invalid ITEMID (functions, callbacks or filters)

#define MPX_ORIG                1   // original medipix2
#define MPX_MXR                 2   // medipix mxr 2.0

typedef unsigned int DEVID;             // type for accessing medipix device (device "handle" or "identifikator")
typedef unsigned int * ITEMID;          // type for accessing registered functions, callbacks or filters
typedef unsigned int FRAMEID;

// function plugin type that can be registered in Manager through function of type AddFuncType
// function has to return 0 on success, non zero on error
typedef int (*PluginFuncType)(byte *in, byte *out);

typedef int CBPARAM;                            // type of parameter for callback functions registered in manager
typedef void (*CallbackFuncType)(CBPARAM par);  // callback type that can be registered in Manager

typedef unsigned int MENUFUNCPAR;               // type of parameter for menu functions registered in tray menu
typedef void (*MenuFuncType)(MENUFUNCPAR par);  // type of function that can be registered through function of type AddMenuType

typedef BOOL (*MsgHandlerType)(LPMSG msg);      // message handler type for registering for win message loop

// function of this type can be registered by plugin to offer custom frame attribute
// these registered functions will be called after frame acquisition to get actual attribute value for current frame
// [in] hint - custom value
// [out] data - buffer for attribute value(s)
// [in/out] size - size of data buffer, if size of supplied buffer is not big enough, size should contain required size after return
typedef int (*GetFrameAttribType)(int hint, byte *data, int *size);

// structure describing one parameter of plugin function
//
typedef struct _ExtParamInfo
{
    Data_Types type;                    // parameter type
    int count;                          // count (for array of type)
    char description[DESC_LENGTH];      // description string
} ExtParamInfo;


// structure describing plugin function registered in Manager
//
typedef struct _ExtFunctionInfo
{
    PluginFuncType func;                        // pointer to registered function
    int paramInCount;                           // number of input parameters
    int paramOutCount;                          // number of output parameters
    ExtParamInfo paramsInfoIn[MAX_PARAM];       // input parameters infos
    ExtParamInfo paramsInfoOut[MAX_PARAM];      // output parameters infos
    char pluginName[NAME_LENGTH];               // plugin name
    char functionName[NAME_LENGTH];             // function name
    char description[DESC_LENGTH];              // function description
    DWORD flags;                                // reserved
} ExtFunctionInfo;


// structure describing plugin function registered in Manager
// no nested structures (problems in matlab)
//
typedef struct _ExtFunctionInfoMatlab
{
    PluginFuncType func;                        // pointer to registered function
    
    //input parameters
    //
    int paramInCount;                           // nubmer of input parameters
    Data_Types typesIn[MAX_PARAM];              // parameters types
    int countsIn[MAX_PARAM];                    // counts (for array of type)
    char descrIn[MAX_PARAM][DESC_LENGTH];       // descriptions string

    // output parameters
    //
    int paramOutCount;                           // number of output parameters
    Data_Types typesOut[MAX_PARAM];              // parameters types
    int countsOut[MAX_PARAM];                    // counts (for array of type)
    char descrOut[MAX_PARAM][DESC_LENGTH];       // descriptions string

    char pluginName[NAME_LENGTH];                // plugin name
    char functionName[NAME_LENGTH];              // function name
    char description[DESC_LENGTH];               // function description
    DWORD flags;                                 // reserved
} ExtFunctionInfoMatlab;

// structure describing plugin callback event registered in Manager
//
typedef struct _ExtCBEventInfo
{
    char pluginName[NAME_LENGTH];               // plugin name
    char cbEventName[NAME_LENGTH];              // callback event name
    char description[DESC_LENGTH];              // callback event description
} ExtCBEventInfo;

// structure describing custom frame attribute provided by plugin (template for attribute)
//
typedef struct _ExtFrameAttribInfo
{
    char pluginName[NAME_LENGTH];   // plugin which provides attribute value
    char attribName[NAME_LENGTH];   // name of attribute
    char description[DESC_LENGTH];  // description
    Data_Types type;                // type of attribute
    int minCount;                   // minimal count that should be allocated
    int hint;                       // hint for GetFrameAttribType function
    GetFrameAttribType func;        // get attribute function
} ExtFrameAttribInfo;

// structure describing custom frame attribute provided by plugin
//
typedef struct _ExtFrameAttrib
{
    char name[NAME_LENGTH];         // name of attribute
    char desc[DESC_LENGTH];          // description of attribute
    Data_Types type;                // type of attribute
    int size;                       // size of data in bytes
} ExtFrameAttrib;


// flags for functions logErrorTab, logErrorStr, 
//
#define MGRLOG_GET_SYSERROR 1       // get last error and makes it part of logged error string (to simplify error reporting when WINAPI calls fail)
#define MGRLOG_SHOW_MSGBOX  2       // show messagebox with error

// error login functions types
// [in] hInstance - HANDLE to DLL module (instance) by which is called
// [in] flags - (MGRLOG_GET_SYSERROR and/or MGRLOG_SHOW_MSGBOX)
// [in] msgID - resource ID in string table in DLL resources (accessed via hInstance handle), this string can have "printf-like" format
// [in] format - "printf-like" format string
// [in] ... - ellipsis (printf-like)
typedef int (*MgrLogErrorTabType)(HINSTANCE hInstance, UINT msgID, UINT flags, ...);
typedef int (*MgrLogErrorStrType)(HINSTANCE hInstance, const char *format, UINT flags, ...);

// shows/hides log window error login functions types
// [in] show - TRUE to show log window, FALSE to hide it
typedef int (*MgrLogShowWindowType)(BOOL show);

// shows/hides windows with list of items managed by manager
// [in] show - TRUE to show window, FALSE to hide it
typedef int (*MgrManagedItemsShowWindowType)(BOOL show);

// function for registering plugin in Manager
// [in] fileName - fileNamePath to DLL
typedef int (*MgrAddPluginType)(const char *fileName);

// function type for adding item in Loader menu
// [in] itemName - name of item in menu
// [in] menuPath - sequence of submenus where menuitem will be placed, submenus are separated by pipe (|) e.g. "Special Plugins|Filteres"
// [in] func - pointer to function that will be called when user clicks on menuitem
// [in] param - parameter which will be given to function 'func'
// [out] menuItemID - pointer to ID which will be assigned to this menuitem, this can be 0 (NULL) if caller is not interested in this value (this ID is used for removing menuitem from menu)
// [in] flags - reserved flags, use 0
typedef int (*MgrAddMenuItemType)(const char *pluginName, const char *itemName, const char *menuPath, MenuFuncType func, MENUFUNCPAR param, DWORD *menuItemID, DWORD flags);

// function type for adding item in Loader menu to submenu corresponding to Medipix device identified by devID
// [in] devID - device ID of medipix
// [in] itemName - name of item in menu
// [in] menuPath - sequence of submenus where menuitem will be placed, submenus are separated by pipe (|) e.g. "Special Plugins|Filteres", these submenus will be placed in submenu correspnding to spedified mpx device
// [in] func - pointer to function that will be called when user clicks on menuitem
// [in] param - parameter which will be given to function 'func'
// [out] menuItemID - pointer to ID which will be assigned to this menuitem, this can be 0 (NULL) if caller is not interested in this value (this ID is used for removing menuitem from menu)
// [in] flags - reserved flags, use 0
typedef int (*MgrAddMpxMenuItemType)(DEVID devID, const char *pluginName, const char *itemName, const char *menuPath, MenuFuncType func, MENUFUNCPAR param, DWORD *menuItemID, DWORD flags);

// function type for removing item in Loader menu
// [in] menuItemID - ID of item that will be removed
typedef int (*MgrRemoveMenuItemType)(DWORD menuItemID);

// [in] devID - device ID of medipix
// [out] menuPath - path to "menu folder" of mpx device
typedef int (*MgrGetMpxMenuPathType)(DEVID devID, char menuPath[MENU_LENGTH]);

// types of function for browsing of items in tray menu
// to enumerate all menu items MgrGetFirstMenuItem has to be called first, 
// return value 0 means that "name" contains path name of first menu item (e.g. path1|path2|someitem)
// use the itemID in subsequent calls of MgrGetNextMenuItem, call it repeatedly until return value != 0
// [in/out] itemID - pointer to "browse handle", do not change its value
// [out] name - path name of menu item, used for call in MgrCallMenuItemByName()
typedef int (*MgrGetFirstMenuItemType)(ITEMID *itemID, char name[MENU_LENGTH]);
typedef int (*MgrGetNextMenuItemType)(ITEMID *itemID, char name[MENU_LENGTH]);

// type of function for calling menu item
// [in] name - path name of menu item
typedef int (*MgrCallMenuItemByNameType)(const char *name);

// type of function for calling menu item
// [in] itemID - ID of menu item
typedef int (*MgrCallMenuItemByIDType)(ITEMID itemID);


// type of function for adding own handler for win messages (pretranslation of win msgs, support for MFC plugins)
// [in] handler - message handler function
typedef void (*MgrAddMsgHandlerType)(MsgHandlerType handler);

// type of function for removing (unregistering) handler
// [in] handler - message handler function that should be unregistered
typedef void (*MgrRemoveHandlerType)(MsgHandlerType handler);

// function type for registering plugin function in Manager
// [in] funcInfo - this structure has to be filled to give manager all informations about function
typedef int (*MgrAddFuncItemType)(ExtFunctionInfo *funcInfo);

// function type for unregistering plugin function in Manager
// [in] pluginName - name of plugin containing function that should be removed
// [in] functionName - name of function that should be unregistered
typedef int (*MgrRemoveFuncItemType)(const char *pluginName, const char *functionName);

// types of function for browsing of registered functions
// to enumerate all registered functions mgrGetRegFirstFunc has to be called first, 
// return value 0 means that funcInfo contains information about registered function, 
// use the funcID in subsequent calls of mgrGetRegNextFunc, call it repeatedly until return value != 0
// [in/out] funcID - pointer to "browse handle", do not change its value
// [out] funcInfo - this structure will be filled by manager to provide all informations about registered function
typedef int (*MgrGetRegFirstFuncType)(ITEMID *funcID, ExtFunctionInfo *funcInfo);
typedef int (*MgrGetRegNextFuncType)(ITEMID *funcID, ExtFunctionInfo *funcInfo);

// function type for registering plugin function in Manager (version for convenient usage from Matlab)
// [in] funcInfo - this structure has to be filled to give manager all informations about function
typedef int (*MgrAddFuncItemMatlabType)(ExtFunctionInfoMatlab *funcInfo);

// types of function for browsing of registered functions (version for convenient usage from Matlab)
// to enumerate all registered functions mgrGetRegFirstFunc has to be called first, 
// return value 0 means that funcInfo contains information about registered function, 
// use the funcID in subsequent calls of mgrGetRegNextFunc, call it repeatedly until return value != 0
// [in/out] funcID - pointer to "browse handle", do not change its value
// [out] funcInfo - this structure will be filled by manager to provide all informations about registered function
typedef int (*MgrGetRegFirstFuncMatlabType)(ITEMID *funcID, ExtFunctionInfoMatlab *funcInfo);
typedef int (*MgrGetRegNextFuncMatlabType)(ITEMID *funcID, ExtFunctionInfoMatlab *funcInfo);

// type of function for getting function info via plugin.functionName
// [in] pluginName - name of plugin where function resides
// [in] functionName - name of function we are interested in
// [out] extFuncInfo - this structure will be filled by manager to provide all informations about registered function
typedef int (*MgrGetFuncItemType)(const char *pluginName, const char *functionName, ExtFunctionInfo *extFuncInfo);

// type of function for calling function info via plugin.functionName
// [in] pluginName - name of plugin where function resides
// [in] functionName - name of function we are interested in
// [in] in - input parameters for specified function
// [out] out - output parameters for specified function
typedef int (*MgrCallFuncItemType)(const char *pluginName, const char *functionName, byte *in, byte *out);


// function type for registering callback event (by plugin) in Manager
// [in] cbEventInfo - structure describing callback event which will be registered
// [out] eventID - assigned ID, this value should be used for setting cb event
typedef int (*MgrAddCBEventType)(ExtCBEventInfo *cbEventInfo, ITEMID *eventID);

// function type for unregistering callback event (by plugin) in Manager
// [in] pluginName - name of plugin where function resides
// [in] cbEventName - name of callback event that shoud be unregistered
typedef int (*MgrRemoveCBEventType)(const char *pluginName, const char *cbEventName);

// types of function for browsing of registered callback events
// to enumerate all registered events mgrGetRegFirstCBEvent has to be called first, 
// return value 0 means that funcInfo contains information about registered function, 
// use the cbEventID in subsequent calls of mgrGetRegNextCBEvent, call it repeatedly until return value != 0
// [in/out] cbEventID - pointer to "browse handle", do not change its value
// [out] extCBEventInfo - this structure will be filled by manager to provide all informations about registered event
typedef int (*MgrGetRegFirstCBEventType)(ITEMID *cbEventID, ExtCBEventInfo *extCBEventInfo);
typedef int (*MgrGetRegNextCBEventType)(ITEMID *cbEventID, ExtCBEventInfo *extCBEventInfo);

// type of function for setting the registered callback event (Manager then calls registered callback function for this "callback event"
// [in] ID - event ID that should be set
// [in] par - parameter that will be given to callback function registered for this event
typedef int (*MgrSetCBEventType)(ITEMID ID, CBPARAM par);

// type of function for registering to receive notifications about callback event
// [in] pluginName - name of plugin which offers required cb event
// [in] cbEventName - name of event for which we register
// [in] callback - pointer to call back that should be called when event occures
typedef int (*MgrRegisterCallbackType)(const char *pluginName, const char *cbEventName, CallbackFuncType callback);

// type of function for registering to receive notifications about callback event
// [in] pluginName - name of plugin which offers the event
// [in] cbEventName - name of event for which should be registration changed
// [in] callback - callback function which we want remove from "call list" when event occurres
typedef int (*MgrUnregisterCallbackType)(const char *pluginName, const char *cbEventName, CallbackFuncType callback);

// function type for registering custom frame attribute
// [in] attribInfo - this structure has to be filled to give manager required informations
typedef int (*MgrAddFrameAttribType)(ExtFrameAttribInfo *attribInfo);

// function type for unregistering custom frame attribute
// [in] pluginName - name of plugin which registered attribute
// [in] attribName - name of attribName that shoud be unregistered
typedef int (*MgrRemoveFrameAttribType)(const char *pluginName, const char *attribName);


// type of function for creating chain of filters
// [in] pluginName - name of plugin which is creating chain of filters
// [in] filterChainName - name of filter chain
// [out] filterChainID - ID of created filter chain
typedef int (*MgrCreateFilterChainType)(const char *pluginName, const char *filterChainName, ITEMID *filterChainID);

// type of function for adding filter to chain of filters
// [in] filterChainID - ID of filter chain to which the filter will be added
// [in] pluginName - name of plugin which contain filter which will be added
// [in] functionName - name of filter
// [in] order - zero based index of filter in chain (0 add to begining, MAXDWORD - add to end)
typedef int (*MgrAddToFilterChainType)(ITEMID filterChainID, const char *pluginName, const char *functionName, DWORD order);

// type of function for removing filter from chain of filters
// [in] filterChainID - ID of filter chain from which the filter will be removed
// [in] pluginName - name of plugin which contain filter which should be removed
// [in] functionName - name of filter
typedef int (*MgrRemoveFromFilterChainType)(ITEMID filterChainID, const char *pluginName, const char *functionName);

// type of function for getting frame from medipix device filtered by chain of filters
// [in] filterChainID - ID of filter chain which will be used
// [in] devID - medipix device identification
// [out] buffer - ouput buffer for frame
// [in] size - size of buffer (for size-check)
// [in] frameNumber - zero-based index of frame
// [out] errorMsg - buffer for error message, can be NULL if caller does not want use this information
typedef int (*MgrGetFilteredFrameType)(ITEMID filterChainID, DEVID devID, double *buffer, u32 size, u32 frameNumber, char errorMsg[MPXCTRL_MAX_MSGLENGTH]);


// frame control
//


// MPXCTRL FUNCTION TYPES
//

// call this function to initialize MpxCtrl library
typedef int (*MpxCtrlInitializeType)();

// types of functions for enumerating found Medipix devices
// to enumerate all devices mpxCtrlGetFirstMpx has to be called first
// return value 0 means that *devID is valid ID of successfuly initialized Medipix
// use the devID in subsequent calls of mpxCtrlGetNextMpx, call it repeatedly until return value != 0
// [in/out] devID - this device ID uniquely identifies particular Medipix in MpxCtrl functions
// [out] count - after call will contain number of found devices; can be NULL
typedef int (*MpxCtrlGetFirstMpxType)(DEVID *devID, int *count);
typedef int (*MpxCtrlGetNextMpxType)(DEVID *devID);

// type of function for getting number of "hardware info items" for particular Medipix
// "hardware info items" are used to access interface specific variables to obtain or set interface specifice information (e.g. muros clock frequency)
// [in] devID - medipix device identification
// [out] count - number of hw info items of this device
typedef int (*MpxCtrlGetHwInfoCountType)(DEVID devID, int *count);

// obtains selected HwInfoItem structure
// [in] devID - medipix device identification
// [in] index - zero-based index of "hardware info item", should be in <0, count-1> (count is obtained via mpxCtrlGetHwInfoCount)
// [out] infoItem - structure that will contain informations about selected item
// [in/out] dataSize - size of infoItem->data for "raw item data", if the dataSize is not big enough for item data, 
// the function returns error (!=0) and *dataSize will contain required size (to allow caller allocate big enough data buffer)
typedef int (*MpxCtrlGetHwInfoItemType)(DEVID devID, int index, HwInfoItem *infoItem, int *dataSize);

// set selected "hardware info item" data
// [in] devID - medipix device identification
// [in] index - zero-based index of "hardware info item", should be in <0, count-1> (count is obtained via mpxCtrlGetHwInfoCount)
// [in] data - pointer to data
// [in] dataSize - size of data (to allow sizecheck of data for certain hw info item)
typedef int (*MpxCtrlSetHwInfoItemType)(DEVID devID, int index, byte *data, int dataSize);	

// try to locks medipix device for exclusive access
// [in] devID - medipix device identification
// [out] success - TRUE if locking was successful
typedef int (*MpxCtrlTryLockDeviceType)(DEVID devID, BOOL *success);

// release locked device, MUST be called after each successful call to MpxCtrlTryLockDevice
// [in] devID - medipix device identification
typedef int (*MpxCtrlReleaseDeviceType)(DEVID devID);

// [in] devID - medipix device identification
// [out] id - id of thread that currently owns medipix device lock
typedef int (*MpxCtrlGetLockOwnerIDType)(DEVID devID, DWORD *id);

// initialize particular Medipix device with values from medipix configuration file
// [in] devID - medipix device identification
// [in] fileName - filepathname of medipix configuration file, can be NULL, if it is NULL default fileName (based on chipboard ID is used)
typedef int (*MpxCtrlInitMpxDeviceType)(DEVID devID, const char *fileName);

// saves current configuration of particular Medipix device to selected medipix configuration file
// [in] devID - medipix device identification
// [in] fileName - filepathname of medipix configuration file, can be NULL, if it is NULL default fileName (based on chipboard ID is used)
typedef int (*MpxCtrlSaveMpxCfgType)(DEVID devID, const char *fileName);

// load configuration from selected medipix configuration file to internal structures of particular Medipix device
// [in] devID - medipix device identification
// [in] fileName - filepathname of medipix configuration file
typedef int (*MpxCtrlLoadMpxCfgType)(DEVID devID, const char *fileName);


// SET/GET ACQUISITION SETTING
//

// sets polarity of selected Medipix device
// [in] devID - medipix device identification
// [in] positive - TRUE for positive polarity, FALSE for negative
typedef int (*MpxCtrlSetPolarityType)(DEVID devID, BOOL positive);

// gets polarity setting for selected medipix device
// [in] devID - medipix device identification
// [out] positive - TRUE for positive polarity, FALSE for negative
typedef int (*MpxCtrlGetPolarityType)(DEVID devID, BOOL *positive);

// sets charge sharing test for selected Medipix device
// [in] devID - medipix device identification
// [in] enable - TRUE to enable, FALSE to disable
typedef int (*MpxCtrlSetCSTType)(DEVID devID, BOOL enable);

// gets charge sharing test setting for selected medipix device
// [in] devID - medipix device identification
// [out] enabled - TRUE if enabled, FALSE if disabled
typedef int (*MpxCtrlGetCSTType)(DEVID devID, BOOL *enabled);

// sets external shutter for selected Medipix device
// [in] devID - medipix device identification
// [in] enable - TRUE to enable, FALSE to disable
typedef int (*MpxCtrlSetExternalShutterType)(DEVID devID, BOOL enable);

// gets external shutter setting for selected medipix device
// [in] devID - medipix device identification
// [out] enabled - TRUE if enabled, FALSE if disabled
typedef int (*MpxCtrlGetExternalShutterType)(DEVID devID, BOOL *enabled);

#define MPXCTRL_HWTIMER_ENABLE      0       // HW timer enabled
#define MPXCTRL_HWTIMER_DISABLE     1       // HW timer disabled (PC timer used)
#define MPXCTRL_HWTIMER_AUTO        2       // MpxCtrl library decides which timer will be used according to time of measurement
// selects which timer is used for selected Medipix device
// [in] devID - medipix device identification
// [in] state - MPXCTRL_HWTIMER_ENABLE, MPXCTRL_HWTIMER_DISABLE, MPXCTRL_HWTIMER_AUTO
typedef int (*MpxCtrlSetHwTimerType)(DEVID devID, int state);

// gets HW timer setting
// [in] devID - medipix device identification
// [out] state - MPXCTRL_HWTIMER_ENABLE, MPXCTRL_HWTIMER_DISABLE, MPXCTRL_HWTIMER_AUTO (see definitions)
typedef int (*MpxCtrlGetHwTimerType)(DEVID devID, int *state);

// enable spare Fast Shift Registers
// [in] devID - medipix device identification
// [in] enable - TRUE to enable, FALSE to disable
typedef int (*MpxCtrlSetSpareFSRType)(DEVID devID, BOOL enable);

// gets spare fast shift register setting for selected medipix device
// [in] devID - medipix device identification
// [out] enabled - TRUE if enabled, FALSE if disabled
typedef int (*MpxCtrlGetSpareFSRType)(DEVID devID, BOOL *enabled);

// sets all DACs of all chips to default (typical for usage) values, depends on currently set polarity
// [in] devID - medipix device identification
typedef int (*MpxCtrlSetDACsDefaultType)(DEVID devID);

// sets DACs of particular chip of certain device
// [in] devID - medipix device identification
// [in] dacVals - array of DAC values
// [in] size - size of array dacVals (should be number of DACs, 13 original, 14 MXR)
// [in] chipNumber - zero-based index of chip
typedef int (*MpxCtrlSetChipDACsType)(DEVID devID, DACTYPE dacVals[], int size, int chipNumber);

// gets DAC values for one chip of selected medipix device
// [in] devID - medipix device identification
// [out] dacVals - output array for DAC values
// [in] size - size of array dacVals (should be number of DACs, 13 original, 14 MXR)
// [in] chipNumber - zero-based index of chip
typedef int (*MpxCtrlGetChipDACsType)(DEVID devID, DACTYPE dacVals[], int size, int chipNumber);

// sets all chips DACs for certain device
// [in] devID - medipix device identification
// [in] dacVals - array of DAC values, dimension is [numberOfChips][13]
// [in] size - size of dacVal array (has to be numberOfChips*dacCount of current chip (13 original mpx., 14 MXR))
typedef int (*MpxCtrlSetAllChipsDACsType)(DEVID devID, DACTYPE dacVals[], int size);

// gets DAC values for all chips of selected medipix device
// [in] devID - medipix device identification
// [out] dacVals - output array for DAC values, size has to be byte[numberOfChips][13]
// [in] size - size of dacVal array (has to be numberOfChips*dacCount of current chip (13 original mpx., 14 MXR))
typedef int (*MpxCtrlGetAllChipsDACsType)(DEVID devID, DACTYPE dacVals[], int size);

// gets single DAC value of single chip of selected medipix device
// [in] devID - medipix device identification
// [out] dacVal - output for DAC analog value
// [in] dacNumber - number from DACS_ORDER specifying which DAC analog value should be read
// [in] chipNumber - zero-based index of chip
typedef int (*MpxCtrlGetSingleDacAnalogType)(DEVID devID, double *dacVal, int dacNumber, int chipNumber);

// gets single DAC value of all chips of selected medipix device
// [in] devID - medipix device identification
// [out] dacVals - output array for DAC values, size has to be double[numberOfChips]
// [in] dacNumbers - numbers from DACS_ORDER specifying which DAC analog value should be read for each chip
// [in] numberOfChips - number of chips (for size-check)
typedef int (*MpxCtrlGetAllChipsSingleDacAnalogType)(DEVID devID, double dacVals[], int dacNumbers[], int numberOfChips);

// gets analog DAC values of all chips of selected medipix device
// [in] devID - medipix device identification
// [out] dacVals - output array for DAC values, size has to be double[numberOfChips*number of dacs]
// [in] size - size of dacVals array (for size-check)
typedef int (*MpxCtrlGetAllChipsDACsAnalogType)(DEVID devID, double dacVals[], int size);

// sends last set DAC values to all chips of selected medipix device
// [in] devID - medipix device identification
typedef int (*MpxCtrlRefreshDACsType)(DEVID devID);

// sends last set DAC values to all chips of selected medipix device
// [in] devID - medipix device identification
// [out] names - *names is const array of names (const char *) of DACs of medipix chip
// [out] precisions - *precisions is const array of precisions of individual DACs in bits
// [out] size - *size of *names array, (e.g. 13 for normal mpx chip, 12 for MXR)
typedef int (*MpxCtrlGetDACsNamesType)(DEVID devID, const char * const **names, const int **precisions, int *size);

// sets pixels configurations of one chip of selected medipix device
// [in] devID - medipix device identification
// [in] pixCfgs - array of pixels configurations
// [in] chipNumber - zero-based number of selected chip
typedef int (*MpxCtrlSetPixelsCfgType)(DEVID devID, PixelCfg pixCfgs[MATRIX_SIZE], int chipNumber);

// gets pixels configurations of one chip of selected medipix device
// [in] devID - medipix device identification
// [out] pixCfgs - array for pixels configurations
// [in] chipNumber - zero-based number of selected chip
typedef int (*MpxCtrlGetPixelsCfgType)(DEVID devID, PixelCfg pixCfgs[MATRIX_SIZE], int chipNumber);

// resets pixels configurations (nothing masked, no test bits, adj bits 7) of one chip of selected medipix device
// [in] devID - medipix device identification
// [in] chipNumber - zero-based number of selected chip
typedef int (*MpxCtrlResetPixelsCfgType)(DEVID devID, int chipNumber);

// sets pixels configurations of all chips of selected medipix device
// [in] devID - medipix device identification
// [in] pixCfgs - arrays of pixels configurations for all chips, size of array has to be [numberOfChips][MATRIX_SIZE]
// [in] numberOfChips - number of chips of selected device (for size-check)
typedef int (*MpxCtrlSetAllChipsPixelsCfgType)(DEVID devID, PixelCfg pixCfgs[][MATRIX_SIZE], int numberOfChips);

// gets pixels configurations of all chips of selected medipix device
// [in] devID - medipix device identification
// [out] pixCfgs - arrays for pixels configurations for all chips, size of array has to be [numberOfChips][MATRIX_SIZE]
// [in] numberOfChips - number of chips of selected device (for size-check)
typedef int (*MpxCtrlGetAllChipsPixelsCfgType)(DEVID devID, PixelCfg pixCfgs[][MATRIX_SIZE], int numberOfChips);

// resets pixels configurations (nothing masked, no test bits, adj bits 7) of all chips of selected medipix device
// [in] devID - medipix device identification
// [in] numberOfChips - number of chips of selected device (for size-check)
typedef int (*MpxCtrlResetAllChipsPixelsCfgType)(DEVID devID, int numberOfChips);

// sends last set pixels cfgs to all chips of selected medipix device
// [in] devID - medipix device identification
typedef int (*MpxCtrlRefreshPixelsCfgType)(DEVID devID);

// sets acquisition spacing for making acquisition in several steps, in each steps part of pixels are active
// the distance is given by spacing parameter (min 1, max 256)
// [in] devID - medipix device identification
// [in] spacing - spacing values
typedef int (*MpxCtrlSetAcqSpacingType)(DEVID devID, int spacing);

// gets acquisition spacing 
// [in] devID - medipix device identification
// [out] spacing - *spacing contains currently set spacing
typedef int (*MpxCtrlGetAcqSpacingType)(DEVID devID, int *spacing);

// loads pixels configurations (mask bit, test bit, and 3-bit threshold adjustements) from "binary pixels configuration" file
// [in] devID - medipix device identification
// [in] fileName - filenamepath of binary file, if it is NULL default filename based on chipboard ID is used
typedef int (*MpxCtrlLoadPixelsCfgType)(DEVID devID, const char *fileName);

// saves pixels configurations (mask bit, test bit, and 3-bit threshold adjustements) to "binary pixels configuration" file
// [in] devID - medipix device identification
// [in] fileName - filenamepath of binary file
typedef int (*MpxCtrlSavePixelsCfgType)(DEVID devID, const char *fileName);

// loads pixels configurations (mask bit, test bit, and 3-bit threshold adjustements) from several ASCII files
// [in] devID - medipix device identification
// [in] maskBitFile - filenamepath of ASCII file containing mask bit for each pixel (0 masked)
// [in] testBitFile - filenamepath of ASCII file containing test bit for each pixel (0 test bit active)
// [in] lowThsFile - filenamepath of ASCII file containing values corresponding to low threshold adjust bits for each pixel (0 all bits active,..., 7 all not active)
// [in] highThsFile - filenamepath of ASCII file containing values corresponding to high threshold adjust bits for each pixel (0 all bits active,..., 7 all not active)
typedef int (*MpxCtrlLoadPixelsCfgAsciiType)(DEVID devID, const char *maskBitFile, const char *testBitFile, const char *lowThsFile, const char *highThsFile);

// saves pixels configurations (mask bit, test bit, and 3-bit threshold adjustements) to several ASCII files
// [in] devID - medipix device identification
// [in] maskBitFile - filenamepath of ASCII file to which mask bit for each pixel (0 masked) will be written
// [in] testBitFile - filenamepath of ASCII file to which test bit for each pixel (0 test bit active) will be written
// [in] lowThsFile - filenamepath of ASCII file to which values corresponding to low threshold adjust bits for each pixel (0 all bits active,..., 7 all not active) will be written 
// [in] highThsFile - filenamepath of ASCII file to which values corresponding to high threshold adjust bits for each pixel (0 all bits active,..., 7 all not active) will be written 
typedef int (*MpxCtrlSavePixelsCfgAsciiType)(DEVID devID, const char *maskBitFile, const char *testBitFile, const char *lowThsFile, const char *highThsFile);


// ACQUISITION CONTROL
//

// perform acquisition in integral mode
// [in] devID - medipix device identification
// [in] numberOfAcq - acquisition count
// [in] timeOfEachAcq - time of each acquistion in seconds
typedef int (*MpxCtrlPerformIntegralAcqType)(DEVID devID, int numberOfAcq, double timeOfEachAcq);

// perform acquisition in frame mode
// [in] devID - medipix device identification
// [in] numberOfFrames - acquisition count
// [in] timeOfEachAcq - time of each acquistion in seconds
// [in] toFiles - if TRUE each acquisition is written to ascii files before starting next, if FALSE all acquistions are held in memory
// [in] fileName - basename of filename (if toFiles is TRUE), resulting file name is created by appending acquisition number to basename
typedef int (*MpxCtrlPerformFrameAcqType)(DEVID devID, int numberOfFrames, double timeOfEachAcq, BOOL toFiles, const char *fileName);

// perform acquisition in fast (stream) mode
// [in] devID - medipix device identification
// [in] numberOfFrames - acquisition count
// [in] timeOfEachAcq - time of each acquistion in seconds
// [in] toFiles - if TRUE each acquisition is written to binary file as a raw (untransformed) stream before starting next, if FALSE all acquistions are held in memory
// [in] fileName - basename of filename (if toFiles is TRUE), resulting file name is created by appending acquisition number to basename
typedef int (*MpxCtrlPerformFastAcqType)(DEVID devID, int numberOfFrames, double timeOfEachAcq, BOOL toFiles, const char *fileName);

// perform test pulse acquisition
// [in] devID - medipix device identification
// [in] lowVal - low value of pulses
// [in] highVal - high value of pulses
// [in] pulseLength - pulse length in clock cycles
// [in] pulseCount - number of pulses
// [in] manual - controls manual/automatic handling of spacing/ctpr, 0 = automatic, nonzero manual (for MXR for manual control this variable has to contain column test pulse register)
typedef int (*MpxCtrlPerformTestPulseAcqType)(DEVID devID, double lowVal, double highVal, byte pulseLength, u16 pulseCount, u32 manual);

// perform full digital test (write random matrix, read it back and counts number of pixels where values agree)
// [in] devID - medipix device identification
// [out] goodPixels - number fo good pixels
// [in] show - mask of digital test result will bw shown in preview
typedef int (*MpxCtrlPerformDigitalTestType)(DEVID devID, u32 *goodPixels, BOOL show);

// abort operation in progress
// [in] devID - medipix device identification
typedef int (*MpxCtrlAbortOperationType)(DEVID devID);


// BUFFER CONTROL
//

// frees allocated stream buffer for selected medipix device
// [in] devID - medipix device identification
typedef int (*MpxCtrlFreeStreamBuffType)(DEVID devID);

// frees allocated frame buffer for selected medipix device
// [in] devID - medipix device identification
typedef int (*MpxCtrlFreeFrameBuffType)(DEVID devID);

// frees allocated integral buffer for selected medipix device
// [in] devID - medipix device identification
typedef int (*MpxCtrlFreeIntegralBuffType)(DEVID devID);

// frees all allocated buffers for selected medipix device
// [in] devID - medipix device identification
typedef int (*MpxCtrlFreeBuffersType)(DEVID devID);

// copy frame from last acquisition to supplied i16[] buffer, this functionality is availaible for all acquisition modes (even for acquisition to files)
// this function can be used for asynchronous access to buffer (e.g. during acquisition, so buffer for error message should be provided
// [in] devID - medipix device identification
// [out] buffer - supplied buffer for frame
// [in] size - size of buffer (for size-check)
// [in] frameNumber - zero-based index of frame
// [out] errorMsg - buffer for error message, can be NULL if caller does not want use this information
typedef int (*MpxCtrlGetFrame16Type)(DEVID devID, i16 *buffer, u32 size, u32 frameNumber, char errorMsg[MPXCTRL_MAX_MSGLENGTH]);

// copy frame from last acquisition to supplied u32[] buffer, this functionality is availaible for all acquisition modes (even for acquisition to files)
// this function can be used for asynchronous access to buffer (e.g. during acquisition, so buffer for error message should be provided
// [in] devID - medipix device identification
// [out] buffer - supplied buffer for frame
// [in] size - size of buffer (for size-check)
// [in] frameNumber - zero-based index of frame
// [out] errorMsg - buffer for error message, can be NULL if caller does not want use this information
typedef int (*MpxCtrlGetFrame32Type)(DEVID devID, u32 *buffer, u32 size, u32 frameNumber, char errorMsg[MPXCTRL_MAX_MSGLENGTH]);

// copy frame from last acquisition to supplied double[] buffer, this functionality is availaible for all acquisition modes (even for acquisition to files)
// this function can be used for asynchronous access to buffer (e.g. during acquisition, so buffer for error message should be provided
// [in] devID - medipix device identification
// [out] buffer - supplied buffer for frame
// [in] size - size of buffer (for size-check)
// [in] frameNumber - zero-based index of frame
// [out] errorMsg - buffer for error message, can be NULL if caller does not want use this information
typedef int (*MpxCtrlGetFrameDoubleType)(DEVID devID, double *buffer, u32 size, u32 frameNumber, char errorMsg[MPXCTRL_MAX_MSGLENGTH]);

// saves selected frame to file in various formats
// [in] devID - medipix device identification
// [in] fileName - filepathname of file, if output is set to multiple files (see flags parameter) this filename is a base name (final filename contains index suffix)
// [in] frameNumber - zero-based frame index
// [in] flags - bitwise combination of following flags: MPXCTRL_SF_STREAM, MPXCTRL_SF_BINARY, MPXCTRL_SF_ASCII, MPXCTRL_SF_ALLFRAMES, MPXCTRL_SF_ONEFILE
// MPXCTRL_SF_STREAM, MPXCTRL_SF_BINARY and MPXCTRL_SF_ASCII flags are mutually exclusive
#define MPXCTRL_SF_STREAM           1       // save as raw stream (available only for fast acquisition modes)
#define MPXCTRL_SF_BINARY           2       // save frames in binary format (i16 (2bytes) for fast/frame, u32 (4bytes) for integral)
#define MPXCTRL_SF_ASCII            4       // save as ASCII
#define MPXCTRL_SF_ALLFRAMES        8       // save all frames (relevant for fast/frame for more than 1 acq)
#define MPXCTRL_SF_ONEFILE          16      // save all frames to single file (relevant if MPXCTRL_SF_ALLFRAMES is set) otherwise every frame goes to individual file
typedef int (*MpxCtrlSaveFrameType)(DEVID devID, const char *fileName, int frameNumber, u32 flags);


// INFO FUNCTIONS
//

// [in] devID - medipix device identification
// [out] type - type of medipix (MPX_ORIG, MPX_MXR)
typedef int (*MpxCtrlGetMpxTypeType)(DEVID devID, u32 *type);

// [in] devID - medipix device identification
// [out] low - low value of pulses
// [out] high - high value of pulses
typedef int (*MpxCtrlGetTestPulseRangeType)(DEVID devID, double *low, double *high);

// gives basic information about selected medipix device
// [in] devID - medipix device identification
// [out] numberOfChips - number of chips
// [out] numberOfRows - number of rows to which is chips are placed in
// [out] chipBoardID - chipboard ID
// [out] ifaceName - name of hardware interface (muros, usb,... ) which is used
typedef int (*MpxCtrlGetMedipixInfoType)(DEVID devID, int *numberOfChips, int *numberOfRows, char chipBoardID[MPX_MAX_CHBID], char ifaceName[MPX_MAX_IFACENAME]);

// provides information about acquisition (in progress or completed)
// [in] devID - medipix device identification
// [out] acqNumber - zero-based acqusition number
// [out] acqTotalCount - total number of acquisition which was ordered
// [out] acqType - type of acqusition (possible output values - MPXCTRL_ACQTYPE_FRAME, MPXCTRL_ACQTYPE_FAST, MPXCTRL_ACQTYPE_INTEGRAL, MPXCTRL_ACQTYPE_FRAME_FILE, MPXCTRL_ACQTYPE_FAST_FILE)
typedef int (*MpxCtrlGetAcqInfoType)(DEVID devID, int *acqNumber, int *acqTotalCount, int *acqType);
#define MPXCTRL_ACQTYPE_FRAME       0       // frame to memory
#define MPXCTRL_ACQTYPE_FAST        1       // raw stream to memory
#define MPXCTRL_ACQTYPE_INTEGRAL    2       // integral to memory
#define MPXCTRL_ACQTYPE_FRAME_FILE  3       // frames to files
#define MPXCTRL_ACQTYPE_FAST_FILE   4       // raw stream to files


// infoType constants for frame info
// 
typedef enum _FRAME_INFO_TYPES
{
    MPX_FRAMEINFO_TIME  = 0,        // time of acqusition (double), in seconds
    MPX_FRAMEINFO_LAST,
} FRAME_INFO_TYPES;

// known frame attributes names
//
#define FAN_DEVID           "Device ID"
#define FAN_CHIPBOARDID     "Chipboard ID"
#define FAN_IFACENAME       "Iface name"
#define FAN_WIDTH           "Width"
#define FAN_HEIGHT          "Height"
#define FAN_ACQTIME         "Acq time"
#define FAN_CFG_FSR         "FSR"
#define FAN_CFG_POLARITY    "Polarity"
#define FAN_CFG_CST         "CST"
#define FAN_CFG_EXTSHUTTER  "Ext Shutter"
#define FAN_CFG_HWTIMER     "HW timer"
#define FAN_CFG_DACS        "DACs"


// provides information about specified frame (can be used during acquisition)
// [in] devID - medipix device identification
// [in] infoType - specification of type of information (see FRAME_INFO_TYPES)
// [in] frameNumber - zero-based index of frame
// [out] value - output buffer (size should be appropriate for output value)
// [in] size - size of output buffer (for size-check)
// [out] errorMsg - buffer for error message
typedef int (*MpxCtrlGetFrameInfoType)(DEVID devID, int infoType, int frameNumber, byte *value, int size, char errorMsg[MPXCTRL_MAX_MSGLENGTH]);

// gets info/error message for selected medipix device
// MpxCtrl library registeres several callback events (see MPXCTRL_CB_XXX) - one of this is MPXCTRL_CB_INFOMSG
// when important/error condition occurred this event is set and plugins registered for this event can call in callback function mpxCtrlGetInfoMsg function to obtain this info/error message
// [in] devID - medipix device identification
// [out] msgType - info/error code
// [out] msg - pointer to info/error message
typedef int (*MpxCtrlGetInfoMsgType)(DEVID devID, int *msgType, const char **msg);


// structure conataining all relevant functions provided by manager, its description is given in types definitions
// size - sizeof the structure (e.g. for version checks)
//
typedef struct _FuncTableType
{
    int size;
    MgrLogErrorTabType logErrorTab;
    MgrLogErrorStrType logErrorStr;
    MgrLogShowWindowType logShowWindow;
    MgrManagedItemsShowWindowType managedItemsShowWindow;

    MgrAddPluginType addPlugin;

    MgrAddMenuItemType addMenuItem;
    MgrAddMpxMenuItemType addMpxMenuItem;    
    MgrRemoveMenuItemType removeMenuItem;
    MgrGetMpxMenuPathType mgrGetMpxMenuPath;
    MgrGetFirstMenuItemType getFirstMenuItem;
    MgrGetNextMenuItemType getNextMenuItem;
    MgrCallMenuItemByNameType callMenuItemByName;
    MgrCallMenuItemByIDType callMenuItemByID;

    MgrAddMsgHandlerType addMsgHandler;
    MgrRemoveHandlerType removeMsgHandler;

    MgrAddFuncItemType addFuncItem;
    MgrRemoveFuncItemType removeFuncItem;
    MgrGetRegFirstFuncType getRegFirstFunc;
    MgrGetRegNextFuncType getRegNextFunc;
    MgrGetFuncItemType getFuncItem;
    MgrCallFuncItemType callFuncItem;

    MgrAddFuncItemMatlabType addFuncItemMatlab;
    MgrGetRegFirstFuncMatlabType getRegFirstFuncMatlab;
    MgrGetRegNextFuncMatlabType getRegNextFuncMatlab;


    MgrAddCBEventType addCBEvent;
    MgrRemoveCBEventType removeCBEvent;    
    MgrGetRegFirstCBEventType getRegFirstCBEvent;
    MgrGetRegNextCBEventType getRegNextCBEvent;
    MgrSetCBEventType setCBEvent;
    MgrRegisterCallbackType registerCallback;
    MgrUnregisterCallbackType unregisterCallback;

    MgrAddFrameAttribType addFrameAttrib;
    MgrRemoveFrameAttribType removeFrameAttrib;

    MgrCreateFilterChainType createFilterChain;
    MgrAddToFilterChainType addToFilterChain;
    MgrRemoveFromFilterChainType removeFromFilterChain;
    MgrGetFilteredFrameType getFilteredFrame;

    // MPXCTRL FUNCTIONS
    //

    MpxCtrlInitializeType mpxCtrlInitialize;
    MpxCtrlGetFirstMpxType mpxCtrlGetFirstMpx;
    MpxCtrlGetNextMpxType mpxCtrlGetNextMpx;
    MpxCtrlGetHwInfoCountType mpxCtrlGetHwInfoCount;
    MpxCtrlGetHwInfoItemType mpxCtrlGetHwInfoItem;
    MpxCtrlSetHwInfoItemType mpxCtrlSetHwInfoItem;

    //
    MpxCtrlTryLockDeviceType mpxCtrlTryLockDevice;
    MpxCtrlReleaseDeviceType mpxCtrlReleaseDevice;
    MpxCtrlGetLockOwnerIDType mpxCtrlGetLockOwnerID;
    MpxCtrlInitMpxDeviceType mpxCtrlInitMpxDevice;
    MpxCtrlSaveMpxCfgType mpxCtrlSaveMpxCfg;
    MpxCtrlLoadMpxCfgType mpxCtrlLoadMpxCfg;

    // SET/GET ACQUISITION SETTING
    //
    MpxCtrlSetPolarityType mpxCtrlSetPolarity;
    MpxCtrlGetPolarityType mpxCtrlGetPolarity;
    MpxCtrlSetCSTType mpxCtrlSetCST;
    MpxCtrlGetCSTType mpxCtrlGetCST;
    MpxCtrlSetExternalShutterType mpxCtrlSetExternalShutter;
    MpxCtrlGetExternalShutterType mpxCtrlGetExternalShutter;
    MpxCtrlSetHwTimerType mpxCtrlSetHwTimer;
    MpxCtrlGetHwTimerType mpxCtrlGetHwTimer;
    MpxCtrlSetSpareFSRType mpxCtrlSetSpareFSR;
    MpxCtrlGetSpareFSRType mpxCtrlGetSpareFSR;
    MpxCtrlSetDACsDefaultType mpxCtrlSetDACsDefault;
    MpxCtrlSetChipDACsType mpxCtrlSetChipDACs;
    MpxCtrlGetChipDACsType mpxCtrlGetChipDACs;
    MpxCtrlSetAllChipsDACsType mpxCtrlSetAllChipsDACs;
    MpxCtrlGetAllChipsDACsType mpxCtrlGetAllChipsDACs;
    MpxCtrlGetSingleDacAnalogType mpxCtrlGetSingleDacAnalog; 
    MpxCtrlGetAllChipsSingleDacAnalogType mpxCtrlGetAllChipsSingleDacAnalog;
    MpxCtrlGetAllChipsDACsAnalogType mpxCtrlGetAllChipsDACsAnalog;
    MpxCtrlRefreshDACsType mpxCtrlRefreshDACs;
    MpxCtrlGetDACsNamesType mpxCtrlGetDACsNames;
    MpxCtrlSetPixelsCfgType mpxCtrlSetPixelsCfg;
    MpxCtrlGetPixelsCfgType mpxCtrlGetPixelsCfg;
    MpxCtrlResetPixelsCfgType mpxCtrlResetPixelsCfg;
    MpxCtrlSetAllChipsPixelsCfgType mpxCtrlSetAllChipsPixelsCfg;
    MpxCtrlGetAllChipsPixelsCfgType mpxCtrlGetAllChipsPixelsCfg;
    MpxCtrlResetAllChipsPixelsCfgType mpxCtrlResetAllChipsPixelsCfg;
    MpxCtrlRefreshPixelsCfgType mpxCtrlRefreshPixelsCfg;
    MpxCtrlSetAcqSpacingType mpxCtrlSetAcqSpacing;
    MpxCtrlGetAcqSpacingType mpxCtrlGetAcqSpacing;

    MpxCtrlLoadPixelsCfgType mpxCtrlLoadPixelsCfg;
    MpxCtrlSavePixelsCfgType mpxCtrlSavePixelsCfg;
    MpxCtrlLoadPixelsCfgAsciiType mpxCtrlLoadPixelsCfgAscii;
    MpxCtrlSavePixelsCfgAsciiType mpxCtrlSavePixelsCfgAscii;

    // ACQUISITION CONTROL
    //
    MpxCtrlPerformIntegralAcqType mpxCtrlPerformIntegralAcq;
    MpxCtrlPerformFrameAcqType mpxCtrlPerformFrameAcq;
    MpxCtrlPerformFastAcqType mpxCtrlPerformFastAcq;
    MpxCtrlPerformTestPulseAcqType mpxCtrlPerformTestPulseAcq;
    MpxCtrlPerformDigitalTestType mpxCtrlPerformDigitalTest;
    MpxCtrlAbortOperationType mpxCtrlAbortOperation;

    // BUFFER CONTROL
    //
    MpxCtrlFreeStreamBuffType mpxCtrlFreeStreamBuff;
    MpxCtrlFreeFrameBuffType mpxCtrlFreeFrameBuff;
    MpxCtrlFreeIntegralBuffType mpxCtrlFreeIntegralBuff;
    MpxCtrlFreeBuffersType mpxCtrlFreeBuffers;
    MpxCtrlGetFrame16Type mpxCtrlGetFrame16;
    MpxCtrlGetFrame32Type mpxCtrlGetFrame32;
    MpxCtrlGetFrameDoubleType mpxCtrlGetFrameDouble;
    MpxCtrlSaveFrameType mpxCtrlSaveFrame;

    // INFO FUNCTIONS
    //
    MpxCtrlGetMpxTypeType mpxCtrlGetMpxType;
    MpxCtrlGetTestPulseRangeType mpxCtrlGetTestPulseRange;
    MpxCtrlGetMedipixInfoType mpxCtrlGetMedipixInfo;
    MpxCtrlGetAcqInfoType mpxCtrlGetAcqInfo;
    MpxCtrlGetFrameInfoType mpxCtrlGetFrameInfo;
    MpxCtrlGetInfoMsgType mpxCtrlGetInfoMsg;
} FuncTableType;


// every plugin has to contain "InitMpxPlugin" function of this type for initialization
typedef DWORD (*InitPluginType)(const FuncTableType * funcTable);


//==============================================================================================
// Helper Macros:


// Initialisation of a Plugin can be done using following Macro:
// Macro creates global variable mgr which is pointer to table of functions exported by Manager.
// Plugin can call Manager functions using mgr pointer.
// Macro contains declaration of PluginInit function. It have to be defined in c(pp) file.
// Function is called automatically during plugin initialisation by Manager.
#define PLUGIN_INIT \
    DWORD PluginInit();\
    const FuncTableType *mgr = NULL;\
    extern "C" __declspec(dllexport) DWORD InitMpxPlugin(const FuncTableType *funcTable)\
    {\
        mgr = funcTable;\
        return PluginInit();\
    }\
    DWORD PluginInit()

// Extended version of previous MACRO containing DllMain function.
// Macro defines moreover global variable Module containing handle of DLL module.
// Macro contais begining of PluginInit function definition => body have to follow, if it is not desired put semicolon after macro
#define PLUGIN_INIT_EXTENDED \
    HINSTANCE Module=NULL; \
    BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)\
    {\
        switch (ul_reason_for_call)\
        {\
            case DLL_PROCESS_ATTACH:\
                Module = (HINSTANCE) hModule;\
                break;\
        }\
        return TRUE;\
    }\
    PLUGIN_INIT
    

